@@ -18,6 +18,73 @@
 
 #include "lightdm/greeter.h"
 
+/**
+ * SECTION:greeter
+ * @short_description: Make a connection to the LightDM daemon and authenticate users
+ * @include: lightdm.h
+ *
+ * #LightDMGreeter is an object that manages the connection to the LightDM server and provides common greeter functionality.
+ *
+ * An example of a simple greeter:
+ * |[
+ * int main ()
+ * {
+ *     GMainLoop *main_loop;
+ *     LightDMGreeter *greeter
+ * 
+ *     main_loop = g_main_loop_new ();
+ * 
+ *     greeter = lightdm_greeter_new ();
+ *     g_object_connect (greeter, "show-prompt", G_CALLBACK (show_prompt_cb), NULL);
+ *     g_object_connect (greeter, "authentication-complete", G_CALLBACK (authentication_complete_cb), NULL);
+ * 
+ *     // Connect to LightDM daemon
+ *     if (!lightdm_greeter_connect_to_daemon_sync (greeter, NULL))
+ *         return EXIT_FAILURE;
+ * 
+ *     // Start authentication
+ *     lightdm_greeter_authenticate (greeter, NULL);
+ * 
+ *     g_main_loop_run (main_loop);
+ * 
+ *     return EXIT_SUCCESS;
+ * }
+ * 
+ * static void show_prompt_cb (LightDMGreeter *greeter, const char *text, LightDMPromptType type)
+ * {
+ *     // Show the user the message and prompt for some response
+ *     gchar *secret = prompt_user (text, type);
+ * 
+ *     // Give the result to the user
+ *     lightdm_greeter_respond (greeter, response);
+ * }
+ * 
+ * static void authentication_complete_cb (LightDMGreeter *greeter)
+ * {
+ *     // Start the session
+ *     if (!lightdm_greeter_get_is_authenticated (greeter) ||
+ *         !lightdm_greeter_start_session_sync (greeter, NULL))
+ *     {
+ *         // Failed authentication, try again
+ *         lightdm_greeter_authenticate (greeter, NULL);
+ *     }
+ * }
+ * ]|
+ */
+
+/**
+ * LightDMGreeter:
+ *
+ * #LightDMGreeter is an opaque data structure and can only be accessed
+ * using the provided functions.
+ */
+
+/**
+ * LightDMGreeterClass:
+ *
+ * Class structure for #LightDMGreeter.
+ */
+
 G_DEFINE_QUARK (lightdm_greeter_error, lightdm_greeter_error)
 
 enum {
@@ -50,6 +117,9 @@ static guint signals[LAST_SIGNAL] = { 0 };
 
 typedef struct
 {
+    /* API version the daemon is using */
+    guint32 api_version;
+
     /* TRUE if the daemon can reuse this greeter */
     gboolean resettable;
 
@@ -101,6 +171,7 @@ G_DEFINE_TYPE (LightDMGreeter, lightdm_greeter, G_TYPE_OBJECT);
 
 #define HEADER_SIZE 8
 #define MAX_MESSAGE_LENGTH 1024
+#define API_VERSION 1
 
 /* Messages from the greeter to the server */
 typedef enum
@@ -126,6 +197,7 @@ typedef enum
     SERVER_MESSAGE_SHARED_DIR_RESULT,
     SERVER_MESSAGE_IDLE,
     SERVER_MESSAGE_RESET,
+    SERVER_MESSAGE_CONNECTED_V2,
 } ServerMessage;
 
 /* Request sent to server */
@@ -515,30 +587,57 @@ send_message (LightDMGreeter *greeter, guint8 *message, gsize message_length, GE
 }
 
 static void
-handle_connected (LightDMGreeter *greeter, guint8 *message, gsize message_length, gsize *offset)
+handle_connected (LightDMGreeter *greeter, gboolean v2, guint8 *message, gsize message_length, gsize *offset)
 {
     LightDMGreeterPrivate *priv = GET_PRIVATE (greeter);
-    gchar *version;
-    GString *hint_string;
+    GString *debug_string;
     int timeout;
     Request *request;
 
-    version = read_string (message, message_length, offset);
-    hint_string = g_string_new ("");
-    while (*offset < message_length)
+    debug_string = g_string_new ("Connected");
+    if (v2)
     {
-        gchar *name, *value;
+        guint32 i, n_env;
+        gchar *version;
 
-        name = read_string (message, message_length, offset);
-        value = read_string (message, message_length, offset);
-        g_hash_table_insert (priv->hints, name, value);
-        g_string_append_printf (hint_string, " %s=%s", name, value);
+        priv->api_version = read_int (message, message_length, offset);
+        g_string_append_printf (debug_string, " api=%u", priv->api_version);
+        version = read_string (message, message_length, offset);
+        g_string_append_printf (debug_string, " version=%s", version);
+        g_free (version);
+        n_env = read_int (message, message_length, offset);
+        for (i = 0; i < n_env; i++)
+        {
+            gchar *name, *value;
+
+            name = read_string (message, message_length, offset);
+            value = read_string (message, message_length, offset);
+            g_hash_table_insert (priv->hints, name, value);
+            g_string_append_printf (debug_string, " %s=%s", name, value);
+        }
+    }
+    else
+    {
+        gchar *version;
+
+        priv->api_version = 0;
+        version = read_string (message, message_length, offset);
+        g_string_append_printf (debug_string, " version=%s", version);
+        g_free (version);
+        while (*offset < message_length)
+        {
+            gchar *name, *value;
+
+            name = read_string (message, message_length, offset);
+            value = read_string (message, message_length, offset);
+            g_hash_table_insert (priv->hints, name, value);
+            g_string_append_printf (debug_string, " %s=%s", name, value);
+        }
     }
 
     priv->connected = TRUE;
-    g_debug ("Connected version=%s%s", version, hint_string->str);
-    g_free (version);
-    g_string_free (hint_string, TRUE);
+    g_debug ("%s", debug_string->str);
+    g_string_free (debug_string, TRUE);
 
     /* Set timeout for default login */
     timeout = lightdm_greeter_get_autologin_timeout_hint (greeter);
@@ -754,7 +853,7 @@ handle_message (LightDMGreeter *greeter, guint8 *message, gsize message_length)
     switch (id)
     {
     case SERVER_MESSAGE_CONNECTED:
-        handle_connected (greeter, message, message_length, &offset);
+        handle_connected (greeter, FALSE, message, message_length, &offset);
         break;
     case SERVER_MESSAGE_PROMPT_AUTHENTICATION:
         handle_prompt_authentication (greeter, message, message_length, &offset);
@@ -774,6 +873,9 @@ handle_message (LightDMGreeter *greeter, guint8 *message, gsize message_length)
     case SERVER_MESSAGE_RESET:
         handle_reset (greeter, message, message_length, &offset);
         break;
+    case SERVER_MESSAGE_CONNECTED_V2:
+        handle_connected (greeter, TRUE, message, message_length, &offset);
+        break;
     default:
         g_warning ("Unknown message from server: %d", id);
         break;
@@ -890,9 +992,10 @@ send_connect (LightDMGreeter *greeter, gboolean resettable, GError **error)
     gsize offset = 0;
 
     g_debug ("Connecting to display manager...");
-    return write_header (message, MAX_MESSAGE_LENGTH, GREETER_MESSAGE_CONNECT, string_length (VERSION) + int_length (), &offset, error) &&
+    return write_header (message, MAX_MESSAGE_LENGTH, GREETER_MESSAGE_CONNECT, string_length (VERSION) + int_length () * 2, &offset, error) &&
            write_string (message, MAX_MESSAGE_LENGTH, VERSION, &offset, error) &&
            write_int (message, MAX_MESSAGE_LENGTH, resettable ? 1 : 0, &offset, error) &&
+           write_int (message, MAX_MESSAGE_LENGTH, API_VERSION, &offset, error) &&
            send_message (greeter, message, offset, error);
 }
 
@@ -2088,7 +2191,7 @@ lightdm_greeter_class_init (LightDMGreeterClass *klass)
      * @greeter: A #LightDMGreeter
      *
      * The ::timed-login signal gets emitted when the automatic login timer has expired.
-     * The application should then call lightdm_greeter_login().
+     * The application should then call lightdm_greeter_authenticate_autologin().
      **/
     signals[AUTOLOGIN_TIMER_EXPIRED] =
         g_signal_new (LIGHTDM_GREETER_SIGNAL_AUTOLOGIN_TIMER_EXPIRED,
